#ifndef COMMON_H
#define COMMON_H

#include <stdio.h>
#include <unistd.h>
#include <time.h>
#include <utmp.h>

#define MAX_USERS		256
#define MAX_PROCS		(MAX_USERS * 4)
#define EXPIRE(f)		((f) & 0x01)
#define PRIORITY(f)		(((f) & 0x0e) >> 1)
#define EXPIRE_DELETE		0x00
#define EXPIRE_UNSUBSCRIBE	0x01
#define min(x,y)		((x) < (y) ? (x) : (y))
#define max(x,y)		((x) > (y) ? (x) : (y))

typedef unsigned char		byte;
typedef unsigned int		word;
typedef unsigned long long	dword;

typedef struct {
  uid_t		uid		__attribute__ ((packed));
  byte		flags		__attribute__ ((packed));
  word		phNoArea       	__attribute__ ((packed));
  word		phNoLocal	__attribute__ ((packed));
  int		tLeft		__attribute__ ((packed));
  int		tLimit		__attribute__ ((packed));
  int		sLeft		__attribute__ ((packed));
  int		sLimit		__attribute__ ((packed));
  time_t	subExpire	__attribute__ ((packed));
  byte		subFlags	__attribute__ ((packed));
  int		subSlimit	__attribute__ ((packed));
  int		subTlimit	__attribute__ ((packed));
  dword		PPPsBytes      	__attribute__ ((packed));
  dword		PPPtBytes	__attribute__ ((packed));
  time_t	expire		__attribute__ ((packed));
  time_t	last		__attribute__ ((packed));
} UserRec;	/* sizeof(UserRec) = 64 */

typedef struct {
  byte		flags		__attribute__ ((packed));
  word		phNoArea       	__attribute__ ((packed));
  word		phNoLocal      	__attribute__ ((packed));
  time_t	expire		__attribute__ ((packed));
} BanRec;

typedef struct {
  pid_t		pid;
  uid_t		uid;
  ushort	tty;
  char		name[16];
} ProcRec;

typedef struct {
  uid_t		uid;
  char		tty[UT_LINESIZE];
  time_t	time;
} LoginRec;

void		errQuit(char *format, ...);
void		perrQuit(char *name);
int		runCommand(char *path, ...);
time_t		addTime(char *s);
void		procList(int *nProcs, ProcRec *procRec);
int		allLinesBusy(int nProcs, ProcRec *procRec, int devBase, int nLines);
void		userList(int *nLogins, LoginRec *loginRec);
int		userLoggedIn(int nLogins, LoginRec *loginRec, uid_t uid);
int		getToken(char *d, char *s, int *idx);
void		fCopy(char *spath, char *dpath);
uid_t		uidFromLogin(char *login);
char		*loginFromUID(char *login, uid_t uid);
void		userFileOpen();
void		userFileClose();
int		userFileRead(UserRec *ur);
int		userFileWrite(UserRec *ur);
void		userFileEdit(UserRec *ur);
int		userFileSearch(UserRec *ur, uid_t uid);
void		banFileOpen();
void		banFileClose();
int		banFileRead(BanRec *br);
int		banFileWrite(BanRec *br);
int		banFileSearch(BanRec *br, word phNoArea, word phNoLocal);
int		phNoToWords(word *phNoArea, word *phNoLocal, char *phNo, int phNoDigits);
char		*wordsToPhNo(char *phNo, word phNoArea, word phNoLocal, char *phNoAreaFormat, char *phNoLocalFormat);
int		stringSum(char *s);
void		bytes2ASCII(char *str, unsigned long long bytes);

extern const char	*configFilePath;
extern const char	*userFilePath;
extern FILE		*userFile;
extern const char	*banFilePath;
extern FILE		*banFile;

#endif
