#include <assert.h>
#include <string.h>
#include <unistd.h>
#include "common.h"

void	processConfig();
void	bytes2ASCII(char *str, unsigned long long bytes);

char	phNoAreaFormat[16];
char	phNoLocalFormat[16];

void main(int argc, char **argv) {
  uid_t uid;
  UserRec ur;
  char str[256], str2[256];
  if (argc != 2)
    errQuit("usage: user_data <login>");
  uid = uidFromLogin(argv[1]);
  if (uid == (uid_t)-1)
    errQuit("user does not exist: %s", argv[1]);
  userFileOpen();
  if (userFileSearch(&ur, uid))
    errQuit("user not found: %s", argv[1]);
  printf("login:     %s\n", argv[1]);
  printf("phone:     ");
  if (ur.phNoLocal) {
    processConfig();
    printf("%s\n", wordsToPhNo(str, ur.phNoArea, ur.phNoLocal, phNoAreaFormat, phNoLocalFormat));
  } else printf("N/A\n");
  printf("priority:  %d\n", (int)PRIORITY(ur.flags));
  printf("time:      %d / %d\n", ur.tLeft, ur.tLimit);
  printf("session:   %d / %d\n", ur.sLeft, ur.sLimit);
  bytes2ASCII(str, ur.PPPsBytes); bytes2ASCII(str2, ur.PPPtBytes);
  printf("PPP data:  %s / %s\n", str, str2);
  printf("expiry:    ");
  if (ur.expire) {
    if (EXPIRE(ur.flags) == EXPIRE_DELETE) printf("DELETE");
    else printf("UNSUBSCRIBE");
    strcpy(str, ctime(&ur.expire)); str[strlen(str) - 1] = '\0';
    printf(" in %.2f days [%s]\n", (float)(ur.expire - time(NULL)) / (24 * 60 * 60), str);    
  } else printf("N/A\n");
  strcpy(str, ctime(&ur.last)); str[strlen(str) - 1] = '\0';
  printf("last on:   -%.2f days [%s]\n", (float)(time(NULL) - ur.last) / (24 * 60 * 60), str);
  userFileClose();
}

void processConfig() {
  int li;
  char line[256], token[256];
  FILE *f;
  if ((f = fopen(configFilePath, "rt")) == NULL)
    errQuit("couldn't open config file");
  while (fgets(line, 256, f) != NULL) {
    li = 0;
    if (getToken(token, line, &li)) continue;
    if (!strcasecmp(token, "PhNoAreaFormat")) {
      if (getToken(token, line, &li)) continue;
      strcpy(phNoAreaFormat, token);
      while (!getToken(token, line, &li)) {
	strcat(phNoAreaFormat, " ");
	strcat(phNoAreaFormat, token);
      }
    } else if (!strcasecmp(token, "PhNoLocalFormat")) {
      if (getToken(token, line, &li)) continue;
      strcpy(phNoLocalFormat, token);
      while (!getToken(token, line, &li)) {
	strcat(phNoLocalFormat, " ");
	strcat(phNoLocalFormat, token);
      }
    }
  }
  fclose(f);
}
