#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>
#include "common.h"

#define		MSG_ALREADY_ONLINE	0
#define		MSG_BUSY		1
#define		MSG_NO_TIME		2
#define		MSG_SESSION_WAIT	3

void		processConfig();
int		devNumFromName(char *devName);
void		printFile(char *path);

const char	*msgPath[4] = { LIB"/user_login.already_online", LIB"/user_login.busy", LIB"/user_login.no_time", LIB"/user_login.session_wait" };

int		optReturnDelay;
int		optGuestTime = 15, optGuestPriority = 4;
int		optSmartBoot, optSmartBootOne;
char		*line0Name;
int		multiBase, nLines;
int		nLogins;
LoginRec	loginRec[MAX_USERS];
int		nProcs;
ProcRec		procRec[MAX_PROCS];

void main() {
  int i, msg, found = 0;
  uid_t uid = getuid();
  UserRec ur, ur2;
  int sLeft;
  uid_t guestUID = uidFromLogin("guest");
  int tty = devNumFromName(ttyname(STDIN_FILENO) + 5);
  int allBusy;
  processConfig();
  userFileOpen();
  if (uid == guestUID) {
    userList(&nLogins, loginRec);
    for (i = 0; i < nLogins && devNumFromName(loginRec[i].tty) != tty; i++);
    if (i == nLogins) {
      msg = -1;
      goto reject;
    }
    ur.flags = optGuestPriority << 1;
    ur.sLeft = optGuestTime - ((time(NULL) - loginRec[i].time) / 60);
    goto guest;
  }
  if (userFileSearch(&ur, uid)) goto accept;
  if (!ur.tLimit) {
    msg = MSG_NO_TIME;
    goto reject; 
  }
  userList(&nLogins, loginRec);
  for (i = 0; i < nLogins; i++)
    if (loginRec[i].uid == uid) {
      found++;
      if (time(NULL) - loginRec[i].time >= 60) {
	msg = MSG_ALREADY_ONLINE;
	goto reject;
      }
    }
  if (found != 1) {
    msg = MSG_ALREADY_ONLINE;
    goto reject;
  }
  if (ur.sLeft <= (int)(0.10 * ur.sLimit) && time(NULL) - ur.last < optReturnDelay * 60) {
    msg = MSG_SESSION_WAIT;
    goto reject;
  }
  ur.PPPsBytes = 0;
  ur.sLeft = min(ur.sLimit, ur.tLeft);
  time(&ur.last);
  userFileEdit(&ur);
guest:
  procList(&nProcs, procRec);
  allBusy = allLinesBusy(nProcs, procRec, multiBase, nLines);
  if (ur.sLeft > 0 && PRIORITY(ur.flags)) goto accept;
  /* no time left, or 0 priority */
  if (optSmartBoot) {
    if (!allBusy) goto accept;
    /* all lines are busy */
    if (!optSmartBootOne) {
      msg = MSG_NO_TIME;
      goto reject;
    }
  } else {
    if (ur.sLeft <= 0) {
      msg = MSG_NO_TIME;
      goto reject;
    }
  }
  /*
    If we get this far, then:

      o The user has no time, or 0 priority
      o Smart-Boot is enabled
      o Smart-Boot-One is enabled
      o All lines are busy

    -OR-

      o The user has 0 priority
      o Smart-Boot is disabled
    
    The decision is non-trivial -- the user could potentially get booted off the
    system the next time user_updated does an update.  Now a check is done to
    see whether or not that would happen.  If it would happen, we will boot the
    user off now.  There is no point (except perhaps to indulge a warped sense
    of humour) in allowing a user on, only to boot them off in < 60 seconds.  Of
    course, that can still happen despite the most strenuous checking, since
    users may come and go at any time, and user_updated's behaviour is very much
    dependent on utmp's contents...
   */
  rewind(userFile);
  while (!userFileRead(&ur2)) {
    if (ur2.uid == uid) continue;
    for (i = 0; i < nProcs && ur.uid != procRec[i].uid; i++);
    if (i == nProcs) continue;
    if (optSmartBoot) {
      if (PRIORITY(ur2.flags) < PRIORITY(ur.flags) || (PRIORITY(ur2.flags) == PRIORITY(ur.flags) && ur2.sLeft < ur.sLeft))
	goto accept;
    } else if (ur2.sLeft < ur.sLeft) goto accept;
  }
  for (i = 0; i < nLogins; i++) {
    if (devNumFromName(loginRec[i].tty) == tty) continue;
    if (loginRec[i].uid != guestUID) continue;
    sLeft = (optGuestTime * 60 - (time(NULL) - loginRec[i].time)) / 60 + 1;
    if (optSmartBoot) {
      if (optGuestPriority < PRIORITY(ur.flags) || (optGuestPriority == PRIORITY(ur.flags) && sLeft < ur.sLeft))
	goto accept;
    } else if (sLeft < ur.sLeft) goto accept;
  }
  if (!PRIORITY(ur.flags)) msg = MSG_BUSY;
  else msg = MSG_NO_TIME;
  goto reject;
accept:
  userFileClose();
  exit(0);
reject:
  userFileClose();
  if (msg >= 0) printFile((char*)msgPath[msg]);
  else printf("Login disallowed -- try again later.\n");
  sleep(2);
  exit(1);
}

void processConfig() {
  int li;
  char line[256], token[256], devPath[UT_LINESIZE];
  struct stat _stat;
  FILE *f;
  if ((f = fopen(configFilePath, "rt")) == NULL)
    perrQuit("fopen");
  while (fgets(line, 256, f) != NULL) {
    li = 0;
    if (getToken(token, line, &li)) continue;
    if (!strcasecmp(token, "Devices")) {
      if (getToken(token, line, &li)) continue;
      line0Name = strdup(token);
      sprintf(devPath, "/dev/%s", line0Name);
      if (stat(devPath, &_stat) < 0)
	perrQuit("stat");
      multiBase = _stat.st_rdev;
      if (getToken(token, line, &li)) continue;
      nLines = atoi(token);
    } else if (!strcasecmp(token, "ReturnDelay")) {
      if (getToken(token, line, &li)) continue;
      optReturnDelay = atoi(token);
    } else if (!strcasecmp(token, "GuestTime")) {
      if (getToken(token, line, &li)) continue;
      optGuestTime = atoi(token);
    } else if (!strcasecmp(token, "GuestPriority")) {
      if (getToken(token, line, &li)) continue;
      optGuestPriority = atoi(token);
    } else if (!strcasecmp(token, "SmartBoot")) {
      optSmartBoot = 1;
    } else if (!strcasecmp(token, "SmartBootOne")) {
      optSmartBootOne = 1;
    }
  }
  fclose(f);
}

int devNumFromName(char *devName) {
  struct stat _stat;
  char devPath[UT_LINESIZE];
  sprintf(devPath, "/dev/%s", devName);
  if (stat(devPath, &_stat) < 0)
    perrQuit("stat");
  return _stat.st_rdev;
}

void printFile(char *path) {
  FILE *f = fopen(path, "r");
  char s[256];
  if (!f) perrQuit("fopen");
  while (fgets(s, 256, f)) printf(s);
  fclose(f);
}
